@@ -1,8 +1,8 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmAuthPeerAuthenticator.cc 10934 2020-08-17 04:03:47Z dwm $
+// @(#) $Id: DwmAuthPeerAuthenticator.cc 10973 2020-08-25 02:48:22Z dwm $
 //===========================================================================
-//  Copyright (c) Daniel W. McRobb 2017
+//  Copyright (c) Daniel W. McRobb 2017, 2020
 //  All rights reserved.
 //
 //  Redistribution and use in source and binary forms, with or without
@@ -62,8 +62,9 @@ extern "C" {
 #include "DwmSysLogger.hh"
 #include "DwmAuthPeerAuthenticator.hh"
 #include "DwmAuthSymCrypto.hh"
+#include "DwmAuthUtils.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwmAuth/tags/libDwmAuth-0.3.4/src/DwmAuthPeerAuthenticator.cc 10934 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwmAuth/tags/libDwmAuth-0.3.5/src/DwmAuthPeerAuthenticator.cc 10973 $");
 
 using namespace std;
 using CryptoPP::PSS;
@@ -150,19 +151,25 @@ namespace Dwm {
       if (fd >= 0) {
         string  mypubEC = to_str(agreement.Public());
         if (IO::Write(fd, mypubEC) == IO::StreamedLength(mypubEC)) {
-          string  theirPubECDH;
-          if (IO::Read(fd, theirPubECDH) > 0) {
-            agreement.TheirPublic(to_SecByteBlock(theirPubECDH));
-            // CryptoPP::SecByteBlock  agreedKeyBB;
-            if (agreement.Agree()) {
-              rc = true;
+          string   theirPubECDH;
+          if (Utils::WaitForBytesReady(fd, 36, chrono::milliseconds(5000))) {
+            ssize_t  bytesRead =
+              Utils::ReadLengthRestrictedString(fd, theirPubECDH, 32);
+            if (bytesRead > 0) {
+              agreement.TheirPublic(to_SecByteBlock(theirPubECDH));
+              if (agreement.Agree()) {
+                rc = true;
+              }
+              else {
+                Syslog(LOG_ERR, "Failed to reach agreement");
+              }
             }
             else {
-              Syslog(LOG_ERR, "Failed to reach agreement");
+              Syslog(LOG_ERR, "Read(%d) failed: %m", fd);
             }
           }
           else {
-            Syslog(LOG_ERR, "Read(%d) failed: %m", fd);
+            Syslog(LOG_ERR, "Peer took to long to send EC exchange");
           }
         }
         else {
@@ -246,17 +253,22 @@ namespace Dwm {
       string  mypubEC = to_str(agreement.Public());
       if (WriteString(s, mypubEC)) {
         string  theirPubECDH;
-        if (ReadString(s, theirPubECDH)) {
-          agreement.TheirPublic(to_SecByteBlock(theirPubECDH));
-          if (agreement.Agree()) {
-            rc = true;
+        if (Utils::WaitForBytesReady(s, 36, chrono::milliseconds(5000))) {
+          if (Utils::ReadLengthRestrictedString(s, theirPubECDH, 32)) {
+            agreement.TheirPublic(to_SecByteBlock(theirPubECDH));
+            if (agreement.Agree()) {
+              rc = true;
+            }
+            else {
+              Syslog(LOG_ERR, "Failed to reach agreement");
+            }
           }
           else {
-            Syslog(LOG_ERR, "Failed to reach agreement");
+            Syslog(LOG_ERR, "Read() failed");
           }
         }
         else {
-          Syslog(LOG_ERR, "Read() failed");
+          Syslog(LOG_ERR, "Peer took to long to send EC exchange");
         }
       }
       else {
@@ -306,13 +318,14 @@ namespace Dwm {
         string idAndSig = BuildIDAndSignature(agreement);
         if (IO::Write(fd, idAndSig)) {
           string  s;
-          if (IO::Read(fd, s) > 0) {
+          auto  endTime = chrono::system_clock::now() + chrono::milliseconds(5000);
+          if (Utils::ReadLengthRestrictedString(fd, s, 2048, endTime) > 0) {
             string  keystr((to_str(agreement.AgreedKey())).substr(0, 16));
             string  ivstr((to_str(agreement.AgreedKey())).substr(16, 16));
             istringstream  is(s);
-            if (IO::Read(is, theirIdCrypted)) {
+            if (Utils::ReadLengthRestrictedString(is, theirIdCrypted, 2044)) {
               theirId = SymCrypto::Decrypt(keystr, ivstr, theirIdCrypted);
-              if (IO::Read(is, theirSig)) {
+              if (Utils::ReadLengthRestrictedString(is, theirSig, 2044)) {
                 rc = true;
               }
               else {
@@ -348,7 +361,8 @@ namespace Dwm {
       string idAndSig = BuildIDAndSignature(agreement);
       if (WriteString(sck, idAndSig)) {
         string  s;
-        if (ReadString(sck, s)) {
+        auto  endTime = chrono::system_clock::now() + chrono::milliseconds(5000);
+        if (Utils::ReadLengthRestrictedString(sck, s, 2048, endTime)) {
           string  keystr((to_str(agreement.AgreedKey())).substr(0, 16));
           string  ivstr((to_str(agreement.AgreedKey())).substr(16, 16));
           istringstream  is(s);
