@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: TestDwmAuthPeerAuthenticator.cc 10617 2020-05-02 21:18:04Z dwm $
+// @(#) $Id: TestDwmAuthPeerAuthenticator.cc 10973 2020-08-25 02:48:22Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -63,7 +63,7 @@ extern "C" {
 #include "DwmAuthECDHAgreement.hh"
 #include "DwmAuthPeerAuthenticator.hh"
 
-static const Dwm::SvnTag  svntag("$DwmPath: dwm/libDwmAuth/tags/libDwmAuth-0.3.4/tests/TestDwmAuthPeerAuthenticator.cc 10617 $");
+static const Dwm::SvnTag  svntag("$DwmPath: dwm/libDwmAuth/tags/libDwmAuth-0.3.5/tests/TestDwmAuthPeerAuthenticator.cc 10973 $");
 
 using namespace std;
 using namespace Dwm;
@@ -110,12 +110,48 @@ void ClientThread(bool preloadKeys)
   return;
 }
 
-static volatile bool g_serverStarted = false;
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+void ClientThreadASIO(bool preloadKeys)
+{
+  namespace ip = boost::asio::ip;
+  using boost::asio::io_context;
+
+  io_context         ioContext;
+  ip::tcp::socket    sck(ioContext);
+  ip::tcp::endpoint  endPoint(ip::address::from_string("127.0.0.1"), 2525);
+  boost::system::error_code  ec;
+  sck.connect(endPoint, ec);
+  if (UnitAssert(! ec)) {
+    if (preloadKeys) {
+      string                   myId;
+      Auth::Ed25519::KeyPair   myKeys;
+      if (UnitAssert(myKeys.Load("./id_ed25519", myId))) {
+        Auth::PeerAuthenticator  peerAuth(myId, myKeys,
+                                          "./known_services");
+        string                   theirid, agreedKey;
+        UnitAssert(peerAuth.Authenticate(sck, theirid, agreedKey));
+        UnitAssert(! theirid.empty());
+        UnitAssert(! agreedKey.empty());
+      }
+    }
+    else {
+      Auth::PeerAuthenticator  peerAuth("./id_ed25519", "./known_services");
+      string                   theirid, agreedKey;
+      UnitAssert(peerAuth.Authenticate(sck, theirid, agreedKey));
+      UnitAssert(! theirid.empty());
+      UnitAssert(! agreedKey.empty());
+    }
+    sck.close();
+  }
+  return;
+}
 
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-void ServerThread(bool preloadKeys)
+void ServerThread(bool preloadKeys, std::atomic<bool> & ready)
 {
   Socket  s;
   if (UnitAssert(s.Open(PF_INET, SOCK_STREAM, 0))) {
@@ -132,7 +168,7 @@ void ServerThread(bool preloadKeys)
     sockAddr.sin_addr.s_addr = INADDR_ANY;
     if (UnitAssert(s.Bind(sockAddr))) {
       if (UnitAssert(s.Listen(10))) {
-        g_serverStarted = true;
+        ready = true;
         Socket  as;
         struct sockaddr_in  clientAddr;
         if (UnitAssert(s.Accept(as, clientAddr))) {
@@ -162,6 +198,52 @@ void ServerThread(bool preloadKeys)
     }
     UnitAssert(s.Close());
   }
+  ready = false;
+  return;
+}
+
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+void ServerThreadASIO(bool preloadKeys, std::atomic<bool> & ready)
+{
+  namespace ip = boost::asio::ip;
+  
+  boost::asio::io_context  ioContext;
+  ip::tcp::acceptor
+    acc(ioContext,
+        ip::tcp::endpoint(ip::address::from_string("127.0.0.1"), 2525),
+        true);
+  boost::asio::socket_base::reuse_address option(true);
+  acc.set_option(option);
+  boost::system::error_code  ec;
+  ip::tcp::socket            sck(ioContext);
+  ip::tcp::endpoint          endPoint;
+  ready = true;
+  acc.accept(sck, endPoint, ec);
+  if (UnitAssert(! ec)) {
+    if (preloadKeys) {
+      string                   myId;
+      Auth::Ed25519::KeyPair  myKeys;
+      if (UnitAssert(myKeys.Load("./id_ed25519", myId))) {
+        Auth::PeerAuthenticator  peerAuth(myId, myKeys,
+                                          "./authorized_keys");
+        string                   theirid, agreedKey;
+        UnitAssert(peerAuth.Authenticate(sck, theirid, agreedKey));
+        UnitAssert(! theirid.empty());
+        UnitAssert(! agreedKey.empty());
+        sck.close();
+      }
+    }
+    else {
+      Auth::PeerAuthenticator  peerAuth("./id_ed25519", "./authorized_keys");
+      string                   theirid, agreedKey;
+      UnitAssert(peerAuth.Authenticate(sck, theirid, agreedKey));
+      UnitAssert(! theirid.empty());
+      UnitAssert(! agreedKey.empty());
+      sck.close();
+    }
+  }
   return;
 }
 
@@ -179,13 +261,37 @@ int main(int argc, char *argv[])
     { true, false }
   };
 
+  std::atomic<bool>  serverReady = false;
   for (auto preload : preloadKeys) {
-    g_serverStarted = false;
-    thread  serverThread(ServerThread, preload.first);
-    while (! g_serverStarted) {}
+    thread  serverThread(ServerThread, preload.first, std::ref(serverReady));
+    while (! serverReady) {}
     thread  clientThread(ClientThread, preload.second);
     clientThread.join();
     serverThread.join();
+
+    serverReady = false;
+    serverThread = std::thread(ServerThread, preload.first,
+                               std::ref(serverReady));
+    while (! serverReady) {}
+    clientThread = std::thread(ClientThreadASIO, preload.second);
+    clientThread.join();
+    serverThread.join();
+
+    serverReady = false;
+    serverThread = std::thread(ServerThreadASIO, preload.first,
+                               std::ref(serverReady));
+    while (! serverReady) {}
+    clientThread = std::thread(ClientThreadASIO, preload.second);
+    clientThread.join();
+    serverThread.join();
+
+    serverReady = false;
+    serverThread = std::thread(ServerThreadASIO, preload.first,
+                               std::ref(serverReady));
+    while (! serverReady) {}
+    clientThread = std::thread(ClientThread, preload.second);
+    clientThread.join();
+    serverThread.join();
   }
   
   int  rc;
